<?php

namespace App;

use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;

class CommandShow extends Command
{
    public static $defaultName = "show";
    public static $defaultDescription = "Show all backup entries";

    protected function configure(): void
    {
        $this->addArgument(
            'config',
            InputArgument::OPTIONAL,
            'Configuration file',
            "config.yml"
        );
    }

    protected function execute(InputInterface $input, OutputInterface $output): int
    {
        $sio = new SymfonyStyle($input, $output);
        $sio->title('List backup entities');

        try {
            $app = new App($input->getArgument('config'));
        } catch (\Throwable $e) {
            $sio->error('Configuration error: ' . $e->getMessage());
            return Command::FAILURE;
        }

        $sio->table(
            ['Description', 'Source', 'Destination'],
            $app->getConfig()['backup']
        );

        $sio->success("Done");
        return Command::SUCCESS;
    }
}
